#include "MK40N512MD100.h"

void UART5Init()
{
	uint32_t SBR;
	// Odblokowanie zegara UART5
	SIM->SCGC1 |= (1 << SIM_SCGC1_UART5_SHIFT);
	SIM->SCGC5 |= (1 << SIM_SCGC5_PORTE_SHIFT);
	
	// Wybr funkcji linii PTE8 i PTE9 jako TX i RX
	PORTE->PCR[8] = PORT_PCR_MUX(3);
	PORTE->PCR[9] = PORT_PCR_MUX(3);

	// Konfiguracja baudrate'u
	SBR = 162;
	UART5->BDH = (SBR >> 8) & 0x1F;
	UART5->BDL = (SBR & 0xFF);
	UART5->C4 = UART_C4_BRFA(0x18);

	// Bez bitu parzystoci (PE=0, PT - bez znaczenia), 8 bitw danych (M=0)
	UART5->C1 = 0; 

	// Odblokowanie odbiornika (RE=1) i nadajnika (TE=1) oraz przerwania
	UART5->C2 = (1 << UART_C2_RE_SHIFT) | (1 << UART_C2_TE_SHIFT) | (1 << UART_C2_RIE_SHIFT);

	NVIC->ISER[1] |= (1 << 23);
}


// Konfiguracja linii PTA14, PTA15, PTA16, PTA17 jako 
// wyjcia typu Push-Pull
void GPIOInit()
{
	// Odblokowanie zegara portu A
	SIM->SCGC5 |= (1 << SIM_SCGC5_PORTA_SHIFT);

	// Ustawienie funkcji linii na GPIO
	PORTA->PCR[14] = PORT_PCR_MUX(1);
	PORTA->PCR[15] = PORT_PCR_MUX(1);
	PORTA->PCR[16] = PORT_PCR_MUX(1);
	PORTA->PCR[17] = PORT_PCR_MUX(1);

	// Ustawienie linii jako wyjcia
	PTA->PDDR |= (1 << 14) | (1 << 15) | (1 << 16) | (1 << 17);
}

void UART5_RX_TX_IRQHandler()
{
	char inData;
	UART5->S1;
	inData = UART5->D;
	switch (inData)
	{
		case '4':
			PTA->PTOR |= (1 << 14);
			break;	
		case '5':
			PTA->PTOR |= (1 << 15);
			break;
		case '6':
			PTA->PTOR |= (1 << 16);
			break;
		case '7':
			PTA->PTOR |= (1 << 17);
			break;
	}
}

void UART5SendString(char * buffer)
{
	while(*buffer != 0) 
	{
		while (!(UART5->S1 & UART_S1_TC_MASK));
		UART5->D = *(buffer++);
	}
}

void SystemInit()
{
  // Wycz watchdoga
  WDOG->UNLOCK = (uint16_t)0xC520u;     /* Key 1 */
  WDOG->UNLOCK  = (uint16_t)0xD928u;    /* Key 2 */
  WDOG->STCTRLH = (uint16_t)0x01D2u;
}


int main()
{
	GPIOInit();
	UART5Init();
	UART5SendString("\r\n====================================================================\r\n");
	UART5SendString("Freescale Kinetis MK40X256 UART Demo\r\n");
	UART5SendString("====================================================================\r\n");
	UART5SendString("Wylij 4 aby zmieni stan diody D4\r\n");
	UART5SendString("Wylij 5 aby zmieni stan diody D5\r\n");
	UART5SendString("Wylij 6 aby zmieni stan diody D6\r\n");
	UART5SendString("Wylij 7 aby zmieni stan diody D7\r\n");
	UART5SendString("====================================================================\r\n");
	
	while(1)
	{
	}
}